#version 330 core

uniform mat4 mvpMatrix;
uniform mat4 mvMatrix;
uniform mat4 mMatrix;
uniform mat4 vMatrix;

uniform highp sampler2D textureMap2; // height map
uniform vec3 lightPosition_world; // light in world space

uniform float textureScale1; // height map scale factor, e.g. 300
uniform float textureScale2; // ground texture scale factor, e.g. 50

layout(location=0) in vec3 position;
layout(location=13) in vec2 heightUV;

out vec3 viewDirection_eye;		// from vertex to camera in eye space
out vec3 lightDirection_eye;	// from vertex to light in eye space
out vec2 uv;					// texture coordinates for fragment shader
out vec2 posXZ_world;			// x,z coordinates of position in world space

void main(void)
{
	// --- get height:
	vec2 pos_world = (mMatrix * vec4(position, 1.0)).xz;
	posXZ_world = pos_world;
	float height = texture(textureMap2, pos_world / textureScale1).x;

	// --- uv coordinates for fragment shader:
	uv = pos_world / textureScale2;

	// --- calculate position:
	vec4 newPosition = vec4(position, 1.0);
	newPosition.y += height;
	gl_Position = mvpMatrix * newPosition;

	// --- lighting:
	vec3 position_eye = (mvMatrix * newPosition).xyz; // vertex position in eye space
	viewDirection_eye = vec3(0.0, 0.0, 0.0) - position_eye;
	
	vec3 lightPosition_eye = (vMatrix * vec4(lightPosition_world, 1.0)).xyz;
	lightDirection_eye = lightPosition_eye - position_eye;
}
