#version 330 core

//change number of bones according to model
const int NUMBONES = 16;

uniform mat4 mvpMatrix;
uniform mat4 mvMatrix;
uniform mat3 nMatrix;
uniform vec3 lightPosition_world; // light in world space
uniform mat4 bones[NUMBONES];

layout(location=0) in vec3 position;
layout(location=1) in vec2 in_UV;
layout(location=2) in vec3 normal;
layout(location=3) in vec4 boneWeight; 
layout(location=4) in ivec4 boneID; 

out vec2 uv;

out vec3 normal_eye;
out vec3 viewDirection_eye;
out vec3 lightDirection_eye;

void main(void)
{
	// Vertex Skinning
	mat4 boneTransform = bones[boneID[0]] * boneWeight[0];
	boneTransform += bones[boneID[1]] * boneWeight[1];
	boneTransform += bones[boneID[2]] * boneWeight[2];
	boneTransform += bones[boneID[3]] * boneWeight[3];

	vec4 bonePosition = boneTransform * vec4(position, 1.0);
	gl_Position = mvpMatrix * bonePosition;
	
	// UV Interpolation
	uv = in_UV;

	// Blinn-Phong Calculations
	//normal: inverse transpose matrix
	vec3 normal = (mat4(nMatrix) * boneTransform * vec4(normal, 1.0)).xyz;
	normal_eye = normalize(normal);
	vec3 position_eye =  (mvMatrix * bonePosition).xyz;
	viewDirection_eye = vec3(0.0, 0.0, 0.0) -position_eye;
	vec3 lightPosition_eye = (mvMatrix * vec4(lightPosition_world, 1.0)).xyz;
	lightDirection_eye = lightPosition_eye - position_eye;
}